import asyncio
import time
from bleak import BleakScanner, BleakClient

DEVICE_NAME = "MYDEVICE"
OTA_DATA_WRITE_UUID = '00000002-0003-4602-a283-df18eacea968'

async def find_device(name: str):
    dev_return = None
    stop_event = asyncio.Event()
    # Callback function that will be called when a device is discovered
    def detection_callback(device, advertisement_data):
        nonlocal dev_return
        if device.name == name:
            print(f"Found device: {device.name} - {device.address}")
            dev_return = device
            stop_event.set()


    async with BleakScanner(detection_callback=detection_callback):
        await stop_event.wait()

    return dev_return

async def scan_and_connect():
    target_device = await find_device(DEVICE_NAME)


    if target_device:
        async with BleakClient(target_device) as client:
            if client.is_connected:
                print(f"Connected to {target_device}")

                services = client.services
                char = services.get_characteristic(OTA_DATA_WRITE_UUID)
                print("MTU: " + str(char.max_write_without_response_size))
                if char.max_write_without_response_size > 20:
                    packet_size = char.max_write_without_response_size
                else:
                    packet_size = 514

                transfer_time = time.time()  # Start the time
                byte_array = bytearray(packet_size)
                
                for i in range(500):
                    # Update only the first two bytes with the value of `i`
                    byte_array[0] = i & 0xFF          # Lower 8 bits
                    byte_array[1] = (i >> 8) & 0xFF   # Higher 8 bits

                    # Print progress every 100 packets
                    if i % 100 == 0:
                        print(f"{i} out of 500 packets sent.")
                    
                    # Print final message when all packets are sent
                    if i == 499:
                        print("All packets sent.")
                    
                    # Writing to the GATT characteristic
                    await client.write_gatt_char(OTA_DATA_WRITE_UUID, byte_array, response=False)

                elapsed_time = time.time() - transfer_time  # Calculate elapsed time
                print(f"Transfer time: {elapsed_time:.2f} seconds")                 
                
    else:
        print(f"Device {DEVICE_NAME} not found.")

if __name__ == "__main__":
    asyncio.run(scan_and_connect())